@@ -57,9 +57,10 @@ enum cmd_and_opt_values { aNull = 0,
     oKOption	  = 'k',
     oDryRun	  = 'n',
     oOutput	  = 'o',
+    oQuiet	  = 'q',
     oRemote	  = 'r',
     aSign	  = 's',
-    oTextmode	  = 't',
+    oTextmodeShort= 't',
     oUser	  = 'u',
     oVerbose	  = 'v',
     oCompress	  = 'z',
@@ -76,6 +77,7 @@ enum cmd_and_opt_values { aNull = 0,
     aKMode,
     aKModeC,
     aImport,
+    aFastImport,
     aVerify,
     aListKeys,
     aListSigs,
@@ -98,6 +100,7 @@ enum cmd_and_opt_values { aNull = 0,
     aEnArmor,
     aGenRandom,
 
+    oTextmode,
     oFingerprint,
     oAnswerYes,
     oAnswerNo,
@@ -174,6 +177,7 @@ static ARGPARSE_OPTS opts[] = {
     { aExport, "export"          , 256, N_("export keys") },
     { aExportSecret, "export-secret-keys" , 256, "@" },
     { aImport, "import",      256     , N_("import/merge keys")},
+    { aFastImport, "fast-import",  256 , "@"},
     { aListPackets, "list-packets",256,N_("list only the sequence of packets")},
   #ifdef IS_G10MAINT
     { aExportOwnerTrust,
@@ -202,10 +206,12 @@ static ARGPARSE_OPTS opts[] = {
     { oUser, "local-user",2, N_("use this user-id to sign or decrypt")},
     { oRemote, "remote-user", 2, N_("use this user-id for encryption")},
     { oCompress, NULL,	      1, N_("|N|set compress level N (0 disables)") },
+    { oTextmodeShort, NULL,   0, "@"},
     { oTextmode, "textmode",  0, N_("use canonical text mode")},
   #endif
     { oOutput, "output",    2, N_("use as output file")},
     { oVerbose, "verbose",   0, N_("verbose") },
+    { oQuiet,	"quiet",   0, N_("be somewhat more quiet") },
     { oForceV3Sigs, "force-v3-sigs", 0, N_("force v3 signatures") },
  /* { oDryRun, "dry-run",   0, N_("do not make any changes") }, */
     { oBatch, "batch",     0, N_("batch mode: never ask")},
@@ -614,6 +620,7 @@ main( int argc, char **argv )
 	  case aCheckKeys: set_cmd( &cmd, aCheckKeys); break;
 	  case aListPackets: set_cmd( &cmd, aListPackets); break;
 	  case aImport: set_cmd( &cmd, aImport); break;
+	  case aFastImport: set_cmd( &cmd, aFastImport); break;
 	  case aExport: set_cmd( &cmd, aExport); break;
 	  case aListKeys: set_cmd( &cmd, aListKeys); break;
 	  case aListSigs: set_cmd( &cmd, aListSigs); break;
@@ -657,6 +664,7 @@ main( int argc, char **argv )
 
 	  case oArmor: opt.armor = 1; opt.no_armor=0; break;
 	  case oOutput: opt.outfile = pargs.r.ret_str; break;
+	  case oQuiet: opt.quiet = 1; break;
 	  case oVerbose: g10_opt_verbose++;
 		    opt.verbose++; opt.list_sigs=1; break;
 	  case oKOption: set_cmd( &cmd, aKMode ); break;
@@ -728,6 +736,7 @@ main( int argc, char **argv )
 	    sl->next = remusr;
 	    remusr = sl;
 	    break;
+	  case oTextmodeShort: opt.textmode = 2; break;
 	  case oTextmode: opt.textmode=1;  break;
 	  case oUser: /* store the local users */
 	    sl = m_alloc( sizeof *sl + strlen(pargs.r.ret_str));
@@ -833,7 +842,7 @@ main( int argc, char **argv )
 
 
     /* kludge to let -sat generate a clear text signature */
-    if( opt.textmode && !detached_sig && opt.armor && cmd == aSign )
+    if( opt.textmode == 2 && !detached_sig && opt.armor && cmd == aSign )
 	cmd = aClearsign;
 
     if( opt.verbose > 1 )
@@ -1022,14 +1031,15 @@ main( int argc, char **argv )
 	break;
     #endif
 
+      case aFastImport:
       case aImport:
 	if( !argc  ) {
-	    rc = import_keys( NULL );
+	    rc = import_keys( NULL, (cmd == aFastImport) );
 	    if( rc )
 		log_error("import failed: %s\n", g10_errstr(rc) );
 	}
 	for( ; argc; argc--, argv++ ) {
-	    rc = import_keys( *argv );
+	    rc = import_keys( *argv, (cmd == aFastImport) );
 	    if( rc )
 		log_error("import from '%s' failed: %s\n",
 						*argv, g10_errstr(rc) );
