@@ -31,6 +31,10 @@
 
 #define MAX_FINGERPRINT_LEN 20
 
+typedef struct getkey_ctx_s *GETKEY_CTX;
+#ifndef DEFINES_GETKEY_CTX
+struct getkey_ctx_s { char hidden[1]; };
+#endif
 
 /****************
  * A Keyblock is all packets which form an entire certificate;
@@ -46,6 +50,7 @@ struct kbnode_struct {
     PACKET *pkt;
     int flag;
     int private_flag;
+    ulong recno;  /* used while updating the trustdb */
 };
 
 
@@ -69,6 +74,7 @@ struct keyblock_pos_struct {
     int secret;      /* working on a secret keyring */
   #ifdef HAVE_LIBGDBM
     GDBM_FILE dbf;
+    byte keybuf[21];
   #endif
     PACKET *pkt;     /* ditto */
 };
@@ -122,8 +128,13 @@ void set_next_passphrase( const char *s );
 char *get_last_passphrase(void);
 
 /*-- getkey.c --*/
+int classify_user_id( const char *name, u32 *keyid, byte *fprint,
+		      const char **retstr, size_t *retlen );
 int get_pubkey( PKT_public_key *pk, u32 *keyid );
-int get_pubkey_byname( PKT_public_key *pk, const char *name );
+int get_pubkey_byname( GETKEY_CTX *rx, PKT_public_key *pk,
+		       const char *name, KBNODE *ret_keyblock );
+int get_pubkey_next( GETKEY_CTX ctx, PKT_public_key *pk, KBNODE *ret_keyblock );
+void get_pubkey_end( GETKEY_CTX ctx );
 int get_seckey( PKT_secret_key *sk, u32 *keyid );
 int get_pubkey_byfprint( PKT_public_key *pk, const byte *fprint,
 						 size_t fprint_len );
@@ -172,6 +183,10 @@ const char *enum_keyblock_resources( int *sequence, int secret );
 int add_keyblock_resource( const char *resname, int force, int secret );
 const char *keyblock_resource_name( KBPOS *kbpos );
 int get_keyblock_handle( const char *filename, int secret, KBPOS *kbpos );
+int locate_keyblock_by_fpr( KBPOS *kbpos, const byte *fpr,
+					    int fprlen, int secret );
+int locate_keyblock_by_keyid( KBPOS *kbpos, u32 *keyid,
+					    int shortkid, int secret );
 int find_keyblock( PUBKEY_FIND_INFO info, KBPOS *kbpos );
 int find_keyblock_byname( KBPOS *kbpos, const char *username );
 int find_keyblock_bypk( KBPOS *kbpos, PKT_public_key *pk );
