@@ -215,7 +215,7 @@ sign_uids( KBNODE keyblock, STRLIST locusr, int *ret_modified )
     SK_LIST sk_rover = NULL;
     PKT_secret_key *sk = NULL;
     KBNODE node, uidnode;
-    PKT_public_key *primary_pk;
+    PKT_public_key *primary_pk=NULL;
     int select_all = !count_selected_uids(keyblock);
     int upd_trust = 0;
 
@@ -571,10 +571,12 @@ keyedit_menu( const char *username, STRLIST locusr )
 	    tty_printf("\n");
 	    redisplay = 0;
 	}
-	m_free(answer);
-	answer = cpr_get(N_("keyedit.cmd"), _("Command> "));
-	cpr_kill_prompt();
-	trim_spaces(answer);
+	do {
+	    m_free(answer);
+	    answer = cpr_get(N_("keyedit.cmd"), _("Command> "));
+	    cpr_kill_prompt();
+	    trim_spaces(answer);
+	} while( *answer == '#' );
 
 	arg_number = 0;
 	if( !*answer )
@@ -645,7 +647,7 @@ keyedit_menu( const char *username, STRLIST locusr )
 	    }
 	    else
 		tty_printf(_("Key not changed so no update needed.\n"));
-	    rc = update_trust_record( keyblock );
+	    rc = update_trust_record( keyblock, 0, NULL );
 	    if( rc )
 		log_error(_("update of trust db failed: %s\n"),
 			    g10_errstr(rc) );
@@ -703,7 +705,7 @@ keyedit_menu( const char *username, STRLIST locusr )
 		sec_modified = modified = 1;
 		/* must update the trustdb already here, so that preferences
 		 * get listed correctly */
-		rc = update_trust_record( keyblock );
+		rc = update_trust_record( keyblock, 0, NULL );
 		if( rc ) {
 		    log_error(_("update of trust db failed: %s\n"),
 				g10_errstr(rc) );
