@@ -180,11 +180,12 @@ proc_pubkey_enc( CTX c, PACKET *pkt )
     if( is_ELGAMAL(enc->pubkey_algo)
 	|| enc->pubkey_algo == PUBKEY_ALGO_DSA
 	|| is_RSA(enc->pubkey_algo)  ) {
-	m_free(c->dek ); /* paranoid: delete a pending DEK */
-	c->dek = m_alloc_secure( sizeof *c->dek );
-	if( (result = get_session_key( enc, c->dek )) ) {
-	    /* error: delete the DEK */
-	    m_free(c->dek); c->dek = NULL;
+	if ( !c->dek ) {
+	    c->dek = m_alloc_secure( sizeof *c->dek );
+	    if( (result = get_session_key( enc, c->dek )) ) {
+		/* error: delete the DEK */
+		m_free(c->dek); c->dek = NULL;
+	    }
 	}
     }
     else
@@ -716,13 +717,6 @@ do_proc_packets( CTX c, IOBUF a )
     c->iobuf = a;
     init_packet(pkt);
     while( (rc=parse_packet(a, pkt)) != -1 ) {
-	/* cleanup if we have an illegal data structure */
-	if( c->dek && pkt->pkttype != PKT_ENCRYPTED ) {
-	    /* FIXME: do we need to ave it in case we have no secret
-	     * key for one of the next reciepents- we should check it
-	     * here. */
-	    m_free(c->dek); c->dek = NULL; /* burn it */
-	}
 
 	if( rc ) {
 	    free_packet(pkt);
@@ -846,7 +840,7 @@ check_sig_and_print( CTX c, KBNODE node )
     tstr = asctimestamp(sig->timestamp);
     astr = pubkey_algo_to_string( sig->pubkey_algo );
     log_info(_("Signature made %.*s using %s key ID %08lX\n"),
-	    (int)strlen(tstr)-1, tstr, astr? astr: "?", (ulong)sig->keyid[1] );
+	    (int)strlen(tstr), tstr, astr? astr: "?", (ulong)sig->keyid[1] );
 
     rc = do_check_sig(c, node, NULL );
     if( !rc || rc == G10ERR_BAD_SIGN ) {
@@ -886,10 +880,14 @@ proc_tree( CTX c, KBNODE node )
     c->local_id = 0;
     c->trustletter = ' ';
     if( node->pkt->pkttype == PKT_PUBLIC_KEY
-	|| node->pkt->pkttype == PKT_PUBLIC_SUBKEY )
+	|| node->pkt->pkttype == PKT_PUBLIC_SUBKEY ) {
+	merge_keys_and_selfsig( node );
 	list_node( c, node );
-    else if( node->pkt->pkttype == PKT_SECRET_KEY )
+    }
+    else if( node->pkt->pkttype == PKT_SECRET_KEY ) {
+	merge_keys_and_selfsig( node );
 	list_node( c, node );
+    }
     else if( node->pkt->pkttype == PKT_ONEPASS_SIG ) {
 	/* check all signatures */
 	if( !c->have_data ) {
