@@ -125,6 +125,8 @@ const char *strusage( int level );
 
 
 /*-- fileutil.c --*/
+char * make_basename(const char *filepath);
+char * make_dirname(const char *filepath);
 char *make_filename( const char *first_part, ... );
 int compare_filenames( const char *a, const char *b );
 const char *print_fname_stdin( const char *s );
@@ -150,6 +152,8 @@ const char *memistr( const char *buf, size_t buflen, const char *sub );
 char *mem2str( char *, const void *, size_t);
 char *trim_spaces( char *string );
 int string_count_chr( const char *string, int c );
+char *native_to_utf8( const char *string );
+char *utf8_to_native( const char *string );
 
 #define stricmp(a,b) strcasecmp((a),(b))
 
