@@ -39,6 +39,9 @@ add_days_to_timestamp( u32 stamp, u16 days )
     return stamp + days*86400L;
 }
 
+/****************
+ * Note: this function returns GMT
+ */
 const char *
 strtimestamp( u32 stamp )
 {
@@ -52,6 +55,26 @@ strtimestamp( u32 stamp )
     return buffer;
 }
 
+/****************
+ * Note: this function returns local time
+ */
+const char *
+asctimestamp( u32 stamp )
+{
+    static char buffer[50];
+    struct tm *tp;
+    time_t atime = stamp;
+
+    tp = localtime( &atime );
+  #ifdef HAVE_STRFTIME
+    strftime( buffer, DIM(buffer)-1, "%c %Z", tp );
+    buffer[DIM(buffer)-1] = 0;
+  #else
+    mem2str( buffer, asctime(tp), DIM(buffer) );
+  #endif
+    return buffer;
+}
+
 /****************
  * Print a string to FP, but filter all control characters out.
  */
