@@ -67,6 +67,16 @@ static unsigned cur_blocks;
 static int disable_secmem;
 static int show_warning;
 static int no_warning;
+static int suspend_warning;
+
+
+static void
+print_warn()
+{
+    if( !no_warning )
+	log_info(_("Warning: using insecure memory!\n"));
+}
+
 
 static void
 lock_pool( void *p, size_t n )
@@ -78,8 +88,10 @@ lock_pool( void *p, size_t n )
     uid = getuid();
 
   #ifdef HAVE_BROKEN_MLOCK
-    if( uid )
-	err = EPERM;
+    if( uid ) {
+	errno = EPERM;
+	err = errno;
+    }
     else {
 	err = mlock( p, n );
 	if( err && errno )
@@ -148,7 +160,7 @@ init_pool( size_t n)
     }
     #endif
     if( pool == (void*)-1 )
-	log_error("can't mmap pool of %u bytes: %s - using malloc\n",
+	log_info("can't mmap pool of %u bytes: %s - using malloc\n",
 			    (unsigned)poolsize, strerror(errno));
     else {
 	pool_is_mmapped = 1;
@@ -179,19 +191,33 @@ compress_pool(void)
 void
 secmem_set_flags( unsigned flags )
 {
+    int was_susp = suspend_warning;
+
     no_warning = flags & 1;
+    suspend_warning = flags & 2;
+
+    /* and now issue the warning if it is not longer suspended */
+    if( was_susp && !suspend_warning && show_warning ) {
+	show_warning = 0;
+	print_warn();
+    }
 }
 
 unsigned
 secmem_get_flags(void)
 {
-    return no_warning ? 1:0;
+    unsigned flags;
+
+    flags  = no_warning      ? 1:0;
+    flags |= suspend_warning ? 2:0;
+    return flags;
 }
 
 void
 secmem_init( size_t n )
 {
     if( !n ) {
+      #ifndef __MINGW32__
 	uid_t uid;
 
 	disable_secmem=1;
@@ -200,6 +226,7 @@ secmem_init( size_t n )
 	    if( setuid( uid ) )
 		log_fatal("failed to drop setuid\n" );
 	}
+      #endif
     }
     else {
 	if( n < DEFAULT_POOLSIZE )
@@ -220,10 +247,9 @@ secmem_malloc( size_t size )
 
     if( !pool_okay )
 	log_bug("secmem not initialized\n");
-    if( show_warning ) {
+    if( show_warning && !suspend_warning ) {
 	show_warning = 0;
-	if( !no_warning )
-	    log_info(_("Warning: using insecure memory!\n"));
+	print_warn();
     }
 
     /* blocks are always a multiple of 32 */
